//==============================================================================
//
//	W M N E T . H
//
//	wmnet.h,v 1.1 1998/08/25 21:07:57 rneswold Exp
//==============================================================================

#define SAFE_STRCPY(s,d) {				\
	strncpy((s), (d), sizeof(s) - 1);	\
	(s)[sizeof(s) - 1] = '\0';			\
}

#define	G_WIDTH	55

//
//	Project-wide function prototypes...
//
int ifGetData(unsigned, unsigned, unsigned long*, unsigned long*);
int ifInit(void);
int ifIsPromisc(unsigned);
int ifIsRunning(unsigned);
int ifIsUp(unsigned);
char const* ifName(unsigned);
int ifSample(void);
int ifTotal(void);
